@@ -1,27 +1,13 @@
-/*=============================================================================
-
-    This file is part of ARB.
-
-    ARB is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    ARB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with ARB; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
+/*
     Copyright (C) 2012 Fredrik Johansson
 
-******************************************************************************/
+    This file is part of Arb.
+
+    Arb is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef ACB_H
 #define ACB_H
@@ -32,6 +18,7 @@
 #define ACB_INLINE static __inline__
 #endif
 
+#include <stdio.h>
 #include "arf.h"
 #include "arb.h"
 
@@ -61,32 +48,24 @@ acb_init(acb_t x)
     arb_init(acb_imagref(x));
 }
 
+void acb_clear(acb_t x);
+
+acb_ptr _acb_vec_init(slong n);
+void _acb_vec_clear(acb_ptr v, slong n);
+
+ACB_INLINE arb_ptr acb_real_ptr(acb_t z) { return acb_realref(z); }
+ACB_INLINE arb_ptr acb_imag_ptr(acb_t z) { return acb_imagref(z); }
+
 ACB_INLINE void
-acb_clear(acb_t x)
+acb_get_real(arb_t re, const acb_t z)
 {
-    arb_clear(acb_realref(x));
-    arb_clear(acb_imagref(x));
-}
-
-ACB_INLINE acb_ptr
-_acb_vec_init(long n)
-{
-    long i;
-    acb_ptr v = (acb_ptr) flint_malloc(sizeof(acb_struct) * n);
-
-    for (i = 0; i < n; i++)
-        acb_init(v + i);
-
-    return v;
+    arb_set(re, acb_realref(z));
 }
 
 ACB_INLINE void
-_acb_vec_clear(acb_ptr v, long n)
+acb_get_imag(arb_t im, const acb_t z)
 {
-    long i;
-    for (i = 0; i < n; i++)
-        acb_clear(v + i);
-    flint_free(v);
+    arb_set(im, acb_imagref(z));
 }
 
 ACB_INLINE int
@@ -113,6 +92,12 @@ acb_is_int(const acb_t z)
     return arb_is_zero(acb_imagref(z)) && arb_is_int(acb_realref(z));
 }
 
+ACB_INLINE int
+acb_is_int_2exp_si(const acb_t z, slong e)
+{
+    return arb_is_zero(acb_imagref(z)) && arb_is_int_2exp_si(acb_realref(z), e);
+}
+
 ACB_INLINE void
 acb_zero(acb_t z)
 {
@@ -142,14 +127,14 @@ acb_set(acb_t z, const acb_t x)
 }
 
 ACB_INLINE void
-acb_set_round(acb_t z, const acb_t x, long prec)
+acb_set_round(acb_t z, const acb_t x, slong prec)
 {
     arb_set_round(acb_realref(z), acb_realref(x), prec);
     arb_set_round(acb_imagref(z), acb_imagref(x), prec);
 }
 
 ACB_INLINE void
-acb_neg_round(acb_t z, const acb_t x, long prec)
+acb_neg_round(acb_t z, const acb_t x, slong prec)
 {
     arb_neg_round(acb_realref(z), acb_realref(x), prec);
     arb_neg_round(acb_imagref(z), acb_imagref(x), prec);
@@ -169,6 +154,12 @@ acb_equal(const acb_t x, const acb_t y)
            arb_equal(acb_imagref(x), acb_imagref(y));
 }
 
+ACB_INLINE int
+acb_equal_si(const acb_t x, slong y)
+{
+    return arb_equal_si(acb_realref(x), y) && arb_is_zero(acb_imagref(x));
+}
+
 ACB_INLINE int
 acb_eq(const acb_t x, const acb_t y)
 {
@@ -226,12 +217,33 @@ acb_set_ui(acb_t z, ulong c)
 }
 
 ACB_INLINE void
-acb_set_si(acb_t z, long c)
+acb_set_d(acb_t z, double c)
+{
+    arb_set_d(acb_realref(z), c);
+    arb_zero(acb_imagref(z));
+}
+
+ACB_INLINE void
+acb_set_si(acb_t z, slong c)
 {
     arb_set_si(acb_realref(z), c);
     arb_zero(acb_imagref(z));
 }
 
+ACB_INLINE void
+acb_set_si_si(acb_t z, slong x, slong y)
+{
+    arb_set_si(acb_realref(z), x);
+    arb_set_si(acb_imagref(z), y);
+}
+
+ACB_INLINE void 
+acb_set_d_d(acb_t z, double x, double y)
+{
+    arb_set_d(acb_realref(z), x);
+    arb_set_d(acb_imagref(z), y);
+}
+
 ACB_INLINE void
 acb_set_fmpz(acb_t z, const fmpz_t c)
 {
@@ -240,16 +252,25 @@ acb_set_fmpz(acb_t z, const fmpz_t c)
 }
 
 ACB_INLINE void
-acb_set_round_fmpz(acb_t z, const fmpz_t y, long prec)
+acb_set_fmpz_fmpz(acb_t z, const fmpz_t x, const fmpz_t y)
+{
+    arb_set_fmpz(acb_realref(z), x);
+    arb_set_fmpz(acb_imagref(z), y);
+}
+
+ACB_INLINE void
+acb_set_round_fmpz(acb_t z, const fmpz_t y, slong prec)
 {
     arb_set_round_fmpz(acb_realref(z), y, prec);
     arb_zero(acb_imagref(z));
 }
 
+int acb_contains_int(const acb_t x);
+
 int acb_get_unique_fmpz(fmpz_t z, const acb_t x);
 
 ACB_INLINE void
-acb_set_fmpq(acb_t z, const fmpq_t c, long prec)
+acb_set_fmpq(acb_t z, const fmpq_t c, slong prec)
 {
     arb_set_fmpq(acb_realref(z), c, prec);
     arb_zero(acb_imagref(z));
@@ -263,7 +284,14 @@ acb_set_arb(acb_t z, const arb_t c)
 }
 
 ACB_INLINE void
-acb_set_round_arb(acb_t z, const arb_t x, long prec)
+acb_set_arb_arb(acb_t z, const arb_t x, const arb_t y)
+{
+    arb_set(acb_realref(z), x);
+    arb_set(acb_imagref(z), y);
+}
+
+ACB_INLINE void
+acb_set_round_arb(acb_t z, const arb_t x, slong prec)
 {
     arb_set_round(acb_realref(z), x, prec);
     arb_zero(acb_imagref(z));
@@ -283,7 +311,6 @@ acb_add_error_arf(acb_t x, const arf_t err)
     arb_add_error_arf(acb_imagref(x), err);
 }
 
-/* TODO: document */
 ACB_INLINE void
 acb_add_error_mag(acb_t x, const mag_t err)
 {
@@ -296,7 +323,7 @@ void acb_get_mag(mag_t z, const acb_t x);
 void acb_get_mag_lower(mag_t z, const acb_t x);
 
 ACB_INLINE void
-acb_get_abs_ubound_arf(arf_t u, const acb_t z, long prec)
+acb_get_abs_ubound_arf(arf_t u, const acb_t z, slong prec)
 {
     if (arb_is_zero(acb_imagref(z)))
     {
@@ -324,7 +351,7 @@ acb_get_abs_ubound_arf(arf_t u, const acb_t z, long prec)
 }
 
 ACB_INLINE void
-acb_get_abs_lbound_arf(arf_t u, const acb_t z, long prec)
+acb_get_abs_lbound_arf(arf_t u, const acb_t z, slong prec)
 {
     if (arb_is_zero(acb_imagref(z)))
     {
@@ -352,7 +379,7 @@ acb_get_abs_lbound_arf(arf_t u, const acb_t z, long prec)
 }
 
 ACB_INLINE void
-acb_get_rad_ubound_arf(arf_t u, const acb_t z, long prec)
+acb_get_rad_ubound_arf(arf_t u, const acb_t z, slong prec)
 {
     /* fixme: this bound is very sloppy */
 
@@ -364,59 +391,84 @@ acb_get_rad_ubound_arf(arf_t u, const acb_t z, long prec)
     arf_mul_2exp_si(u, u, 1);
 }
 
-void acb_arg(arb_t r, const acb_t z, long prec);
+ACB_INLINE void
+acb_union(acb_t res, const acb_t x, const acb_t y, slong prec)
+{
+    arb_union(acb_realref(res), acb_realref(x), acb_realref(y), prec);
+    arb_union(acb_imagref(res), acb_imagref(x), acb_imagref(y), prec);
+}
+
+void acb_arg(arb_t r, const acb_t z, slong prec);
+
+void acb_sgn(acb_t res, const acb_t z, slong prec);
+
+void acb_csgn(arb_t res, const acb_t z);
 
 ACB_INLINE void
-acb_add(acb_t z, const acb_t x, const acb_t y, long prec)
+acb_add(acb_t z, const acb_t x, const acb_t y, slong prec)
 {
     arb_add(acb_realref(z), acb_realref(x), acb_realref(y), prec);
     arb_add(acb_imagref(z), acb_imagref(x), acb_imagref(y), prec);
 }
 
 ACB_INLINE void
-acb_sub(acb_t z, const acb_t x, const acb_t y, long prec)
+acb_sub(acb_t z, const acb_t x, const acb_t y, slong prec)
 {
     arb_sub(acb_realref(z), acb_realref(x), acb_realref(y), prec);
     arb_sub(acb_imagref(z), acb_imagref(x), acb_imagref(y), prec);
 }
 
 ACB_INLINE void
-acb_add_ui(acb_t z, const acb_t x, ulong c, long prec)
+acb_add_si(acb_t z, const acb_t x, ulong c, slong prec)
+{
+    arb_add_si(acb_realref(z), acb_realref(x), c, prec);
+    arb_set_round(acb_imagref(z), acb_imagref(x), prec);
+}
+
+ACB_INLINE void
+acb_add_ui(acb_t z, const acb_t x, ulong c, slong prec)
 {
     arb_add_ui(acb_realref(z), acb_realref(x), c, prec);
     arb_set_round(acb_imagref(z), acb_imagref(x), prec);
 }
 
 ACB_INLINE void
-acb_sub_ui(acb_t z, const acb_t x, ulong c, long prec)
+acb_sub_si(acb_t z, const acb_t x, ulong c, slong prec)
+{
+    arb_sub_si(acb_realref(z), acb_realref(x), c, prec);
+    arb_set_round(acb_imagref(z), acb_imagref(x), prec);
+}
+
+ACB_INLINE void
+acb_sub_ui(acb_t z, const acb_t x, ulong c, slong prec)
 {
     arb_sub_ui(acb_realref(z), acb_realref(x), c, prec);
     arb_set_round(acb_imagref(z), acb_imagref(x), prec);
 }
 
 ACB_INLINE void
-acb_add_fmpz(acb_t z, const acb_t x, const fmpz_t y, long prec)
+acb_add_fmpz(acb_t z, const acb_t x, const fmpz_t y, slong prec)
 {
     arb_add_fmpz(acb_realref(z), acb_realref(x), y, prec);
     arb_set_round(acb_imagref(z), acb_imagref(x), prec);
 }
 
 ACB_INLINE void
-acb_add_arb(acb_t z, const acb_t x, const arb_t y, long prec)
+acb_add_arb(acb_t z, const acb_t x, const arb_t y, slong prec)
 {
     arb_add(acb_realref(z), acb_realref(x), y, prec);
     arb_set_round(acb_imagref(z), acb_imagref(x), prec);
 }
 
 ACB_INLINE void
-acb_sub_fmpz(acb_t z, const acb_t x, const fmpz_t y, long prec)
+acb_sub_fmpz(acb_t z, const acb_t x, const fmpz_t y, slong prec)
 {
     arb_sub_fmpz(acb_realref(z), acb_realref(x), y, prec);
     arb_set_round(acb_imagref(z), acb_imagref(x), prec);
 }
 
 ACB_INLINE void
-acb_sub_arb(acb_t z, const acb_t x, const arb_t y, long prec)
+acb_sub_arb(acb_t z, const acb_t x, const arb_t y, slong prec)
 {
     arb_sub(acb_realref(z), acb_realref(x), y, prec);
     arb_set_round(acb_imagref(z), acb_imagref(x), prec);
@@ -437,34 +489,34 @@ acb_conj(acb_t z, const acb_t x)
 }
 
 ACB_INLINE void
-acb_abs(arb_t u, const acb_t z, long prec)
+acb_abs(arb_t u, const acb_t z, slong prec)
 {
     arb_hypot(u, acb_realref(z), acb_imagref(z), prec);
 }
 
 ACB_INLINE void
-acb_mul_ui(acb_t z, const acb_t x, ulong y, long prec)
+acb_mul_ui(acb_t z, const acb_t x, ulong y, slong prec)
 {
     arb_mul_ui(acb_realref(z), acb_realref(x), y, prec);
     arb_mul_ui(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_mul_si(acb_t z, const acb_t x, long y, long prec)
+acb_mul_si(acb_t z, const acb_t x, slong y, slong prec)
 {
     arb_mul_si(acb_realref(z), acb_realref(x), y, prec);
     arb_mul_si(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_mul_fmpz(acb_t z, const acb_t x, const fmpz_t y, long prec)
+acb_mul_fmpz(acb_t z, const acb_t x, const fmpz_t y, slong prec)
 {
     arb_mul_fmpz(acb_realref(z), acb_realref(x), y, prec);
     arb_mul_fmpz(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_mul_arb(acb_t z, const acb_t x, const arb_t y, long prec)
+acb_mul_arb(acb_t z, const acb_t x, const arb_t y, slong prec)
 {
     arb_mul(acb_realref(z), acb_realref(x), y, prec);
     arb_mul(acb_imagref(z), acb_imagref(x), y, prec);
@@ -500,12 +552,12 @@ acb_div_onei(acb_t z, const acb_t x)
     }
 }
 
-void acb_mul(acb_t z, const acb_t x, const acb_t y, long prec);
+void acb_mul(acb_t z, const acb_t x, const acb_t y, slong prec);
 
-void acb_mul_naive(acb_t z, const acb_t x, const acb_t y, long prec);
+void acb_mul_naive(acb_t z, const acb_t x, const acb_t y, slong prec);
 
 ACB_INLINE void
-acb_mul_2exp_si(acb_t z, const acb_t x, long e)
+acb_mul_2exp_si(acb_t z, const acb_t x, slong e)
 {
     arb_mul_2exp_si(acb_realref(z), acb_realref(x), e);
     arb_mul_2exp_si(acb_imagref(z), acb_imagref(x), e);
@@ -518,126 +570,133 @@ acb_mul_2exp_fmpz(acb_t z, const acb_t x, const fmpz_t c)
     arb_mul_2exp_fmpz(acb_imagref(z), acb_imagref(x), c);
 }
 
-void acb_addmul(acb_t z, const acb_t x, const acb_t y, long prec);
+void acb_addmul(acb_t z, const acb_t x, const acb_t y, slong prec);
 
-void acb_submul(acb_t z, const acb_t x, const acb_t y, long prec);
+void acb_submul(acb_t z, const acb_t x, const acb_t y, slong prec);
 
 ACB_INLINE void
-acb_addmul_ui(acb_t z, const acb_t x, ulong y, long prec)
+acb_addmul_ui(acb_t z, const acb_t x, ulong y, slong prec)
 {
     arb_addmul_ui(acb_realref(z), acb_realref(x), y, prec);
     arb_addmul_ui(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_addmul_si(acb_t z, const acb_t x, long y, long prec)
+acb_addmul_si(acb_t z, const acb_t x, slong y, slong prec)
 {
     arb_addmul_si(acb_realref(z), acb_realref(x), y, prec);
     arb_addmul_si(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_submul_ui(acb_t z, const acb_t x, ulong y, long prec)
+acb_submul_ui(acb_t z, const acb_t x, ulong y, slong prec)
 {
     arb_submul_ui(acb_realref(z), acb_realref(x), y, prec);
     arb_submul_ui(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_submul_si(acb_t z, const acb_t x, long y, long prec)
+acb_submul_si(acb_t z, const acb_t x, slong y, slong prec)
 {
     arb_submul_si(acb_realref(z), acb_realref(x), y, prec);
     arb_submul_si(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_addmul_fmpz(acb_t z, const acb_t x, const fmpz_t y, long prec)
+acb_addmul_fmpz(acb_t z, const acb_t x, const fmpz_t y, slong prec)
 {
     arb_addmul_fmpz(acb_realref(z), acb_realref(x), y, prec);
     arb_addmul_fmpz(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_submul_fmpz(acb_t z, const acb_t x, const fmpz_t y, long prec)
+acb_submul_fmpz(acb_t z, const acb_t x, const fmpz_t y, slong prec)
 {
     arb_submul_fmpz(acb_realref(z), acb_realref(x), y, prec);
     arb_submul_fmpz(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_addmul_arb(acb_t z, const acb_t x, const arb_t y, long prec)
+acb_addmul_arb(acb_t z, const acb_t x, const arb_t y, slong prec)
 {
     arb_addmul(acb_realref(z), acb_realref(x), y, prec);
     arb_addmul(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
 ACB_INLINE void
-acb_submul_arb(acb_t z, const acb_t x, const arb_t y, long prec)
+acb_submul_arb(acb_t z, const acb_t x, const arb_t y, slong prec)
 {
     arb_submul(acb_realref(z), acb_realref(x), y, prec);
     arb_submul(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
-void acb_inv(acb_t z, const acb_t x, long prec);
+void acb_inv(acb_t z, const acb_t x, slong prec);
 
-void acb_div(acb_t z, const acb_t x, const acb_t y, long prec);
+void acb_div(acb_t z, const acb_t x, const acb_t y, slong prec);
 
 ACB_INLINE void
-acb_div_ui(acb_t z, const acb_t x, ulong c, long prec)
+acb_div_ui(acb_t z, const acb_t x, ulong c, slong prec)
 {
     arb_div_ui(acb_realref(z), acb_realref(x), c, prec);
     arb_div_ui(acb_imagref(z), acb_imagref(x), c, prec);
 }
 
 ACB_INLINE void
-acb_div_si(acb_t z, const acb_t x, long c, long prec)
+acb_div_si(acb_t z, const acb_t x, slong c, slong prec)
 {
     arb_div_si(acb_realref(z), acb_realref(x), c, prec);
     arb_div_si(acb_imagref(z), acb_imagref(x), c, prec);
 }
 
 ACB_INLINE void
-acb_div_arb(acb_t z, const acb_t x, const arb_t c, long prec)
+acb_div_arb(acb_t z, const acb_t x, const arb_t c, slong prec)
 {
     arb_div(acb_realref(z), acb_realref(x), c, prec);
     arb_div(acb_imagref(z), acb_imagref(x), c, prec);
 }
 
 ACB_INLINE void
-acb_div_fmpz(acb_t z, const acb_t x, const fmpz_t c, long prec)
+acb_div_fmpz(acb_t z, const acb_t x, const fmpz_t c, slong prec)
 {
     arb_div_fmpz(acb_realref(z), acb_realref(x), c, prec);
     arb_div_fmpz(acb_imagref(z), acb_imagref(x), c, prec);
 }
 
-void acb_cube(acb_t y, const acb_t x, long prec);
-void acb_pow_fmpz(acb_t y, const acb_t b, const fmpz_t e, long prec);
-void acb_pow_ui(acb_t y, const acb_t b, ulong e, long prec);
-void acb_pow_si(acb_t y, const acb_t b, long e, long prec);
+void acb_cube(acb_t y, const acb_t x, slong prec);
+void acb_pow_fmpz(acb_t y, const acb_t b, const fmpz_t e, slong prec);
+void acb_pow_ui(acb_t y, const acb_t b, ulong e, slong prec);
+void acb_pow_si(acb_t y, const acb_t b, slong e, slong prec);
 
 ACB_INLINE void
-acb_const_pi(acb_t x, long prec)
+acb_const_pi(acb_t x, slong prec)
 {
     arb_const_pi(acb_realref(x), prec);
     arb_zero(acb_imagref(x));
 }
 
-void acb_log(acb_t r, const acb_t z, long prec);
-void acb_log1p(acb_t r, const acb_t z, long prec);
-void acb_atan(acb_t r, const acb_t z, long prec);
+void acb_log(acb_t r, const acb_t z, slong prec);
+void acb_log1p(acb_t r, const acb_t z, slong prec);
 
-void acb_exp(acb_t r, const acb_t z, long prec);
-void acb_exp_pi_i(acb_t r, const acb_t z, long prec);
-void acb_exp_invexp(acb_t r, acb_t s, const acb_t z, long prec);
+void acb_exp(acb_t r, const acb_t z, slong prec);
+void acb_exp_pi_i(acb_t r, const acb_t z, slong prec);
+void acb_exp_invexp(acb_t r, acb_t s, const acb_t z, slong prec);
+void acb_expm1(acb_t r, const acb_t z, slong prec);
 
-void acb_sin(acb_t r, const acb_t z, long prec);
-void acb_cos(acb_t r, const acb_t z, long prec);
-void acb_sin_cos(acb_t s, acb_t c, const acb_t z, long prec);
-void acb_tan(acb_t r, const acb_t z, long prec);
-void acb_cot(acb_t r, const acb_t z, long prec);
+void acb_sin(acb_t r, const acb_t z, slong prec);
+void acb_cos(acb_t r, const acb_t z, slong prec);
+void acb_sin_cos(acb_t s, acb_t c, const acb_t z, slong prec);
+void acb_tan(acb_t r, const acb_t z, slong prec);
+void acb_cot(acb_t r, const acb_t z, slong prec);
+
+void acb_asin(acb_t r, const acb_t z, slong prec);
+void acb_acos(acb_t r, const acb_t z, slong prec);
+void acb_atan(acb_t r, const acb_t z, slong prec);
+void acb_asinh(acb_t r, const acb_t z, slong prec);
+void acb_acosh(acb_t r, const acb_t z, slong prec);
+void acb_atanh(acb_t r, const acb_t z, slong prec);
 
 ACB_INLINE void
-acb_sinh(acb_t y, const acb_t x, long prec)
+acb_sinh(acb_t y, const acb_t x, slong prec)
 {
     acb_mul_onei(y, x);
     acb_sin(y, y, prec);
@@ -645,14 +704,14 @@ acb_sinh(acb_t y, const acb_t x, long prec)
 }
 
 ACB_INLINE void
-acb_cosh(acb_t y, const acb_t x, long prec)
+acb_cosh(acb_t y, const acb_t x, slong prec)
 {
     acb_mul_onei(y, x);
     acb_cos(y, y, prec);
 }
 
 ACB_INLINE void
-acb_sinh_cosh(acb_t y, acb_t z, const acb_t x, long prec)
+acb_sinh_cosh(acb_t y, acb_t z, const acb_t x, slong prec)
 {
     acb_mul_onei(y, x);
     acb_sin_cos(y, z, y, prec);
@@ -660,7 +719,7 @@ acb_sinh_cosh(acb_t y, acb_t z, const acb_t x, long prec)
 }
 
 ACB_INLINE void
-acb_tanh(acb_t y, const acb_t x, long prec)
+acb_tanh(acb_t y, const acb_t x, slong prec)
 {
     acb_mul_onei(y, x);
     acb_tan(y, y, prec);
@@ -668,61 +727,109 @@ acb_tanh(acb_t y, const acb_t x, long prec)
 }
 
 ACB_INLINE void
-acb_coth(acb_t y, const acb_t x, long prec)
+acb_coth(acb_t y, const acb_t x, slong prec)
 {
     acb_mul_onei(y, x);
     acb_cot(y, y, prec);
     acb_mul_onei(y, y);
 }
 
-void acb_sin_pi(acb_t r, const acb_t z, long prec);
-void acb_cos_pi(acb_t r, const acb_t z, long prec);
-void acb_sin_cos_pi(acb_t s, acb_t c, const acb_t z, long prec);
-void acb_tan_pi(acb_t r, const acb_t z, long prec);
-void acb_cot_pi(acb_t r, const acb_t z, long prec);
+void acb_sech(acb_t r, const acb_t z, slong prec);
+void acb_csch(acb_t r, const acb_t z, slong prec);
 
-void acb_pow_arb(acb_t z, const acb_t x, const arb_t y, long prec);
-void acb_pow(acb_t r, const acb_t x, const acb_t y, long prec);
+ACB_INLINE void
+acb_sec(acb_t y, const acb_t x, slong prec)
+{
+    acb_mul_onei(y, x);
+    acb_sech(y, y, prec);
+}
 
-void acb_sqrt(acb_t y, const acb_t x, long prec);
-void acb_rsqrt(acb_t y, const acb_t x, long prec);
+ACB_INLINE void
+acb_csc(acb_t y, const acb_t x, slong prec)
+{
+    acb_mul_onei(y, x);
+    acb_csch(y, y, prec);
+    acb_mul_onei(y, y);
+}
 
-void acb_rising_ui_bs(acb_t y, const acb_t x, ulong n, long prec);
-void acb_rising_ui_rs(acb_t y, const acb_t x, ulong n, ulong m, long prec);
-void acb_rising_ui_rec(acb_t y, const acb_t x, ulong n, long prec);
-void acb_rising_ui(acb_t z, const acb_t x, ulong n, long prec);
+void acb_sin_pi(acb_t r, const acb_t z, slong prec);
+void acb_cos_pi(acb_t r, const acb_t z, slong prec);
+void acb_sin_cos_pi(acb_t s, acb_t c, const acb_t z, slong prec);
+void acb_tan_pi(acb_t r, const acb_t z, slong prec);
+void acb_cot_pi(acb_t r, const acb_t z, slong prec);
 
-void acb_rising2_ui_bs(acb_t u, acb_t v, const acb_t x, ulong n, long prec);
-void acb_rising2_ui_rs(acb_t u, acb_t v, const acb_t x, ulong n, ulong m, long prec);
-void acb_rising2_ui(acb_t u, acb_t v, const acb_t x, ulong n, long prec);
+void acb_sinc(acb_t res, const acb_t z, slong prec);
+void acb_sinc_pi(acb_t res, const acb_t z, slong prec);
+
+void acb_pow_arb(acb_t z, const acb_t x, const arb_t y, slong prec);
+void acb_pow(acb_t r, const acb_t x, const acb_t y, slong prec);
+
+void acb_sqrt(acb_t y, const acb_t x, slong prec);
+void acb_rsqrt(acb_t y, const acb_t x, slong prec);
+
+void acb_root_ui(acb_t y, const acb_t x, ulong k, slong prec);
+
+void acb_quadratic_roots_fmpz(acb_t r1, acb_t r2,
+    const fmpz_t a, const fmpz_t b, const fmpz_t c, slong prec);
+
+void acb_chebyshev_t_ui(acb_t a, ulong n, const acb_t x, slong prec);
+void acb_chebyshev_t2_ui(acb_t a, acb_t b, ulong n, const acb_t x, slong prec);
+void acb_chebyshev_u_ui(acb_t a, ulong n, const acb_t x, slong prec);
+void acb_chebyshev_u2_ui(acb_t a, acb_t b, ulong n, const acb_t x, slong prec);
+
+void acb_rising_ui_bs(acb_t y, const acb_t x, ulong n, slong prec);
+void acb_rising_ui_rs(acb_t y, const acb_t x, ulong n, ulong m, slong prec);
+void acb_rising_ui_rec(acb_t y, const acb_t x, ulong n, slong prec);
+void acb_rising_ui(acb_t z, const acb_t x, ulong n, slong prec);
+void acb_rising(acb_t z, const acb_t x, const acb_t n, slong prec);
+
+void acb_rising2_ui_bs(acb_t u, acb_t v, const acb_t x, ulong n, slong prec);
+void acb_rising2_ui_rs(acb_t u, acb_t v, const acb_t x, ulong n, ulong m, slong prec);
+void acb_rising2_ui(acb_t u, acb_t v, const acb_t x, ulong n, slong prec);
 
 void acb_rising_ui_get_mag(mag_t bound, const acb_t s, ulong n);
 
-void acb_gamma(acb_t y, const acb_t x, long prec);
-void acb_rgamma(acb_t y, const acb_t x, long prec);
-void acb_lgamma(acb_t y, const acb_t x, long prec);
-void acb_log_sin_pi(acb_t res, const acb_t z, long prec);
-void acb_digamma(acb_t y, const acb_t x, long prec);
-void acb_zeta(acb_t z, const acb_t s, long prec);
-void acb_hurwitz_zeta(acb_t z, const acb_t s, const acb_t a, long prec);
-void acb_polygamma(acb_t res, const acb_t s, const acb_t z, long prec);
+void acb_gamma(acb_t y, const acb_t x, slong prec);
+void acb_rgamma(acb_t y, const acb_t x, slong prec);
+void acb_lgamma(acb_t y, const acb_t x, slong prec);
+void acb_log_sin_pi(acb_t res, const acb_t z, slong prec);
+void acb_digamma(acb_t y, const acb_t x, slong prec);
+void acb_zeta(acb_t z, const acb_t s, slong prec);
+void acb_hurwitz_zeta(acb_t z, const acb_t s, const acb_t a, slong prec);
+void acb_polygamma(acb_t res, const acb_t s, const acb_t z, slong prec);
 
-void acb_log_barnes_g(acb_t res, const acb_t z, long prec);
-void acb_barnes_g(acb_t res, const acb_t z, long prec);
+void acb_bernoulli_poly_ui(acb_t res, ulong n, const acb_t x, slong prec);
 
-void acb_polylog(acb_t w, const acb_t s, const acb_t z, long prec);
-void acb_polylog_si(acb_t w, long s, const acb_t z, long prec);
+void acb_log_barnes_g(acb_t res, const acb_t z, slong prec);
+void acb_barnes_g(acb_t res, const acb_t z, slong prec);
 
-void acb_agm1(acb_t m, const acb_t z, long prec);
-void acb_agm1_cpx(acb_ptr m, const acb_t z, long len, long prec);
+void acb_polylog(acb_t w, const acb_t s, const acb_t z, slong prec);
+void acb_polylog_si(acb_t w, slong s, const acb_t z, slong prec);
+
+void acb_agm1(acb_t m, const acb_t z, slong prec);
+void acb_agm1_cpx(acb_ptr m, const acb_t z, slong len, slong prec);
+
+#define ACB_LAMBERTW_LEFT 2
+#define ACB_LAMBERTW_MIDDLE 4
+
+void acb_lambertw_asymp(acb_t res, const acb_t z, const fmpz_t k, slong L, slong M, slong prec);
+int acb_lambertw_check_branch(const acb_t w, const fmpz_t k, slong prec);
+void acb_lambertw_bound_deriv(mag_t res, const acb_t z, const acb_t ez1, const fmpz_t k);
+void acb_lambertw(acb_t res, const acb_t z, const fmpz_t k, int flags, slong prec);
+
+ACB_INLINE void
+acb_sqr(acb_t res, const acb_t val, slong prec)
+{
+    acb_mul(res, val, val, prec);
+}
 
 /*
 TBD
 
-void acb_invroot_newton(acb_t r, const acb_t a, ulong m, const acb_t r0, long startprec, long prec);
-void acb_root_exp(acb_t r, const acb_t a, long m, long index, long prec);
-void acb_root_newton(acb_t r, const acb_t a, long m, long index, long prec);
-void acb_root(acb_t r, const acb_t a, long m, long index, long prec);
+void acb_invroot_newton(acb_t r, const acb_t a, ulong m, const acb_t r0, slong startprec, slong prec);
+void acb_root_exp(acb_t r, const acb_t a, slong m, slong index, slong prec);
+void acb_root_newton(acb_t r, const acb_t a, slong m, slong index, slong prec);
+void acb_root(acb_t r, const acb_t a, slong m, slong index, slong prec);
 */
 
 ACB_INLINE int
@@ -738,18 +845,24 @@ acb_indeterminate(acb_t x)
     arb_indeterminate(acb_imagref(x));
 }
 
-ACB_INLINE void
-_acb_vec_zero(acb_ptr A, long n)
+ACB_INLINE acb_ptr
+_acb_vec_entry_ptr(acb_ptr vec, slong i)
 {
-    long i;
+    return vec + i;
+}
+
+ACB_INLINE void
+_acb_vec_zero(acb_ptr A, slong n)
+{
+    slong i;
     for (i = 0; i < n; i++)
         acb_zero(A + i);
 }
 
 ACB_INLINE int
-_acb_vec_is_zero(acb_srcptr vec, long len)
+_acb_vec_is_zero(acb_srcptr vec, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         if (!acb_is_zero(vec + i))
             return 0;
@@ -757,121 +870,121 @@ _acb_vec_is_zero(acb_srcptr vec, long len)
 }
 
 ACB_INLINE void
-_acb_vec_set(acb_ptr res, acb_srcptr vec, long len)
+_acb_vec_set(acb_ptr res, acb_srcptr vec, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_set(res + i, vec + i);
 }
 
 ACB_INLINE void
-_acb_vec_set_round(acb_ptr res, acb_srcptr vec, long len, long prec)
+_acb_vec_set_round(acb_ptr res, acb_srcptr vec, slong len, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_set_round(res + i, vec + i, prec);
 }
 
 ACB_INLINE void
-_acb_vec_neg(acb_ptr res, acb_srcptr vec, long len)
+_acb_vec_neg(acb_ptr res, acb_srcptr vec, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_neg(res + i, vec + i);
 }
 
 ACB_INLINE void
-_acb_vec_add(acb_ptr res, acb_srcptr vec1, acb_srcptr vec2, long len, long prec)
+_acb_vec_add(acb_ptr res, acb_srcptr vec1, acb_srcptr vec2, slong len, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_add(res + i, vec1 + i, vec2 + i, prec);
 }
 
 ACB_INLINE void
-_acb_vec_sub(acb_ptr res, acb_srcptr vec1, acb_srcptr vec2, long len, long prec)
+_acb_vec_sub(acb_ptr res, acb_srcptr vec1, acb_srcptr vec2, slong len, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_sub(res + i, vec1 + i, vec2 + i, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_submul(acb_ptr res, acb_srcptr vec, long len, const acb_t c, long prec)
+_acb_vec_scalar_submul(acb_ptr res, acb_srcptr vec, slong len, const acb_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_submul(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_addmul(acb_ptr res, acb_srcptr vec, long len, const acb_t c, long prec)
+_acb_vec_scalar_addmul(acb_ptr res, acb_srcptr vec, slong len, const acb_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_addmul(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_mul(acb_ptr res, acb_srcptr vec, long len, const acb_t c, long prec)
+_acb_vec_scalar_mul(acb_ptr res, acb_srcptr vec, slong len, const acb_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_mul(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_mul_ui(acb_ptr res, acb_srcptr vec, long len, ulong c, long prec)
+_acb_vec_scalar_mul_ui(acb_ptr res, acb_srcptr vec, slong len, ulong c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_mul_ui(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_mul_2exp_si(acb_ptr res, acb_srcptr vec, long len, long c)
+_acb_vec_scalar_mul_2exp_si(acb_ptr res, acb_srcptr vec, slong len, slong c)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_mul_2exp_si(res + i, vec + i, c);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_mul_onei(acb_ptr res, acb_srcptr vec, long len)
+_acb_vec_scalar_mul_onei(acb_ptr res, acb_srcptr vec, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_mul_onei(res + i, vec + i);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_div_ui(acb_ptr res, acb_srcptr vec, long len, ulong c, long prec)
+_acb_vec_scalar_div_ui(acb_ptr res, acb_srcptr vec, slong len, ulong c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_div_ui(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_div(acb_ptr res, acb_srcptr vec, long len, const acb_t c, long prec)
+_acb_vec_scalar_div(acb_ptr res, acb_srcptr vec, slong len, const acb_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_div(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_mul_arb(acb_ptr res, acb_srcptr vec, long len, const arb_t c, long prec)
+_acb_vec_scalar_mul_arb(acb_ptr res, acb_srcptr vec, slong len, const arb_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_mul_arb(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_div_arb(acb_ptr res, acb_srcptr vec, long len, const arb_t c, long prec)
+_acb_vec_scalar_div_arb(acb_ptr res, acb_srcptr vec, slong len, const arb_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
     {
         arb_div(acb_realref(res + i), acb_realref(vec + i), c, prec);
@@ -880,54 +993,74 @@ _acb_vec_scalar_div_arb(acb_ptr res, acb_srcptr vec, long len, const arb_t c, lo
 }
 
 ACB_INLINE void
-_acb_vec_scalar_mul_fmpz(acb_ptr res, acb_srcptr vec, long len, const fmpz_t c, long prec)
+_acb_vec_scalar_mul_fmpz(acb_ptr res, acb_srcptr vec, slong len, const fmpz_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_mul_fmpz(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-_acb_vec_scalar_div_fmpz(acb_ptr res, acb_srcptr vec, long len, const fmpz_t c, long prec)
+_acb_vec_scalar_div_fmpz(acb_ptr res, acb_srcptr vec, slong len, const fmpz_t c, slong prec)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_div_fmpz(res + i, vec + i, c, prec);
 }
 
 ACB_INLINE void
-acb_print(const acb_t x)
+acb_fprint(FILE * file, const acb_t x)
 {
-    printf("(");
-    arb_print(acb_realref(x));
-    printf(", ");
-    arb_print(acb_imagref(x));
-    printf(")");
+    flint_fprintf(file, "(");
+    arb_fprint(file, acb_realref(x));
+    flint_fprintf(file, ", ");
+    arb_fprint(file, acb_imagref(x));
+    flint_fprintf(file, ")");
 }
 
-void acb_printd(const acb_t z, long digits);
+ACB_INLINE void
+acb_print(const acb_t x)
+{
+    acb_fprint(stdout, x);
+}
 
-void acb_randtest(acb_t z, flint_rand_t state, long prec, long mag_bits);
+void acb_fprintd(FILE * file, const acb_t z, slong digits);
 
-void acb_randtest_special(acb_t z, flint_rand_t state, long prec, long mag_bits);
+ACB_INLINE void
+acb_printd(const acb_t z, slong digits)
+{
+    acb_fprintd(stdout, z, digits);
+}
 
-void acb_randtest_precise(acb_t z, flint_rand_t state, long prec, long mag_bits);
+void acb_fprintn(FILE * fp, const acb_t z, long digits, ulong flags);
 
-void acb_randtest_param(acb_t z, flint_rand_t state, long prec, long mag_bits);
+ACB_INLINE void
+acb_printn(const acb_t x, slong digits, ulong flags)
+{
+    acb_fprintn(stdout, x, digits, flags);
+}
 
-long acb_rel_error_bits(const acb_t x);
+void acb_randtest(acb_t z, flint_rand_t state, slong prec, slong mag_bits);
 
-ARB_INLINE long
+void acb_randtest_special(acb_t z, flint_rand_t state, slong prec, slong mag_bits);
+
+void acb_randtest_precise(acb_t z, flint_rand_t state, slong prec, slong mag_bits);
+
+void acb_randtest_param(acb_t z, flint_rand_t state, slong prec, slong mag_bits);
+
+slong acb_rel_error_bits(const acb_t x);
+
+ACB_INLINE slong
 acb_rel_accuracy_bits(const acb_t x)
 {
     return -acb_rel_error_bits(x);
 }
 
 
-ACB_INLINE long
+ACB_INLINE slong
 acb_bits(const acb_t x)
 {
-    long b1, b2;
+    slong b1, b2;
     b1 = arb_bits(acb_realref(x));
     b2 = arb_bits(acb_imagref(x));
     return FLINT_MAX(b1, b2);
@@ -940,9 +1073,9 @@ acb_is_real(const acb_t x)
 }
 
 ACB_INLINE int
-_acb_vec_is_real(acb_srcptr v, long len)
+_acb_vec_is_real(acb_srcptr v, slong len)
 {
-    long i;
+    slong i;
 
     for (i = 0; i < len; i++)
     {
@@ -953,16 +1086,16 @@ _acb_vec_is_real(acb_srcptr v, long len)
     return 1;
 }
 
-ACB_INLINE long
-_acb_vec_bits(acb_srcptr vec, long len)
+ACB_INLINE slong
+_acb_vec_bits(acb_srcptr vec, slong len)
 {
     return _arb_vec_bits((arb_srcptr) vec, 2 * len);
 }
 
 ACB_INLINE void
-_acb_vec_set_powers(acb_ptr xs, const acb_t x, long len, long prec)
+_acb_vec_set_powers(acb_ptr xs, const acb_t x, slong len, slong prec)
 {
-    long i;
+    slong i;
 
     for (i = 0; i < len; i++)
     {
@@ -978,17 +1111,17 @@ _acb_vec_set_powers(acb_ptr xs, const acb_t x, long len, long prec)
 }
 
 ACB_INLINE void
-_acb_vec_add_error_arf_vec(acb_ptr res, arf_srcptr err, long len)
+_acb_vec_add_error_arf_vec(acb_ptr res, arf_srcptr err, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_add_error_arf(res + i, err + i);
 }
 
 ACB_INLINE void
-_acb_vec_add_error_mag_vec(acb_ptr res, mag_srcptr err, long len)
+_acb_vec_add_error_mag_vec(acb_ptr res, mag_srcptr err, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
     {
         mag_add(arb_radref(acb_realref(res + i)),
@@ -999,23 +1132,23 @@ _acb_vec_add_error_mag_vec(acb_ptr res, mag_srcptr err, long len)
 }
 
 ACB_INLINE void
-_acb_vec_indeterminate(acb_ptr vec, long len)
+_acb_vec_indeterminate(acb_ptr vec, slong len)
 {
     _arb_vec_indeterminate((arb_ptr) vec, 2 * len);
 }
 
 ACB_INLINE void
-_acb_vec_trim(acb_ptr res, acb_srcptr vec, long len)
+_acb_vec_trim(acb_ptr res, acb_srcptr vec, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         acb_trim(res + i, vec + i);
 }
 
-ARB_INLINE int
-_acb_vec_get_unique_fmpz_vec(fmpz * res,  acb_srcptr vec, long len)
+ACB_INLINE int
+_acb_vec_get_unique_fmpz_vec(fmpz * res,  acb_srcptr vec, slong len)
 {
-    long i;
+    slong i;
     for (i = 0; i < len; i++)
         if (!acb_get_unique_fmpz(res + i, vec + i))
             return 0;
@@ -1023,7 +1156,29 @@ _acb_vec_get_unique_fmpz_vec(fmpz * res,  acb_srcptr vec, long len)
 }
 
 /* sort complex numbers in a nice-to-display order */
-void _acb_vec_sort_pretty(acb_ptr vec, long len);
+void _acb_vec_sort_pretty(acb_ptr vec, slong len);
+
+/* roots of unity */
+void acb_unit_root(acb_t res, ulong order, slong prec);
+void _acb_vec_unit_roots(acb_ptr z, slong order, slong len, slong prec);
+
+ACB_INLINE slong
+acb_allocated_bytes(const acb_t x)
+{
+    return arb_allocated_bytes(acb_realref(x)) + arb_allocated_bytes(acb_imagref(x));
+}
+
+ACB_INLINE slong
+_acb_vec_allocated_bytes(acb_srcptr vec, slong len)
+{
+    return _arb_vec_allocated_bytes((arb_srcptr) vec, 2 * len);
+}
+
+ACB_INLINE double
+_acb_vec_estimate_allocated_bytes(slong len, slong prec)
+{
+    return 2 * _arb_vec_estimate_allocated_bytes(len, prec);
+}
 
 #ifdef __cplusplus
 }
