@@ -521,6 +521,12 @@ void arb_acos(arb_t z, const arb_t x, slong prec);
 void arb_atanh(arb_t z, const arb_t x, slong prec);
 void arb_asinh(arb_t z, const arb_t x, slong prec);
 void arb_acosh(arb_t z, const arb_t x, slong prec);
+
+void arb_sec(arb_t res, const arb_t x, slong prec);
+void arb_csc(arb_t res, const arb_t x, slong prec);
+void arb_sech(arb_t res, const arb_t x, slong prec);
+void arb_csch(arb_t res, const arb_t x, slong prec);
+
 void arb_fac_ui(arb_t z, ulong n, slong prec);
 void arb_doublefac_ui(arb_t z, ulong n, slong prec);
 void arb_bin_ui(arb_t z, const arb_t n, ulong k, slong prec);
@@ -937,6 +943,7 @@ void _arb_exp_taylor_rs(mp_ptr y, mp_limb_t * error,
     mp_srcptr x, mp_size_t xn, ulong N);
 
 void arb_exp_arf_bb(arb_t z, const arf_t x, slong prec, int minus_one);
+void arb_exp_arf_rs_generic(arb_t res, const arf_t x, slong prec, int minus_one);
 
 int _arb_get_mpn_fixed_mod_log2(mp_ptr w, fmpz_t q, mp_limb_t * error,
                                                 const arf_t x, mp_size_t wn);
@@ -982,6 +989,10 @@ void _arb_sin_cos_taylor_rs(mp_ptr ysin, mp_ptr ycos,
 int _arb_get_mpn_fixed_mod_pi4(mp_ptr w, fmpz_t q, int * octant,
     mp_limb_t * error, const arf_t x, mp_size_t wn);
 
+void arb_sin_cos_arf_bb(arb_t zsin, arb_t zcos, const arf_t x, slong prec);
+void arb_sin_cos_arf_rs_generic(arb_t res_sin, arb_t res_cos, const arf_t x, slong prec);
+void arb_sin_cos_arf_generic(arb_t res_sin, arb_t res_cos, const arf_t x, slong prec);
+
 ARB_INLINE mp_bitcnt_t
 _arb_mpn_leading_zeros(mp_srcptr d, mp_size_t n)
 {
